@@ -1,10 +1,13 @@
+package Tangram::Storage;
+
 use strict;
 
-package Tangram::Storage;
+use Tangram::Storage::Statement;
+
 use DBI;
 use Carp;
 use Tangram::Core;
-use Carp qw/croak confess/;
+
 use vars qw( %storage_class );
 
 BEGIN {
@@ -192,31 +195,8 @@ sub _open
 		print $Tangram::TRACE "Tangram: $obj is ".($id?"oid $id" : "not in storage")."\n";
 	  }
 	  return undef unless $id;
-      if (defined($self->{objects}{$id})) {
-      # This needs to be very paranoid...
-      # $self->{ids} is $address => $id
-      # $self->{objects} is $id => weak $object
-      #
-      # Consider:
-      # 1) Welcome ob1 at XXXX
-      #    $self->{ids}{XXXX} = 1
-      #    $self->{objects}{1} = weak $ob1
-      # 2) $ob1 goes out of scope
-      #    $self->{ids}{XXXX} = 1
-      #    $self->{objects}{1} = undef
-      # 3) Welcome ob 1 at YYYY
-      #    $self->{ids}{XXXX} = 1
-      #    $self->{ids}{YYYY} = 1
-      #    $self->{objects}{1} = weak $ob1
-      # 4) Create new object ob 2 at XXXX, try to lookup if it has an ID
-      #    Find $self->{ids}{XXXX} = 1
-      #    Find defined($self->{objects}{1})
-      #    Eeek - you could get it wrong now! (By returning 1)
-      #    Must check refaddr($self) eq refaddr($self->{objects}{1})
-      #    Not the same object, need to allocate a new ID!
-          return undef unless (Tangram::refaddr($self->{objects}{$id}) eq $address);
-          return $id;
-      }
+	  # refaddr's can be re-used, but weakrefs are magic :-)
+	  return $id if defined($self->{objects}{$id});
 	  delete $self->{ids}{$address};
 	  delete $self->{objects}{$id};
 	  return undef;
@@ -298,7 +278,7 @@ sub my_cursor
 sub select_data
 {
     my $self = shift;
-    Tangram::Select->new(@_)->execute($self, $self->open_connection());
+    Tangram::Expr::Select->new(@_)->execute($self, $self->open_connection());
 }
 
 sub selectall_arrayref
@@ -309,7 +289,7 @@ sub selectall_arrayref
 sub my_select_data
 {
     my $self = shift;
-    Tangram::Select->new(@_)->execute($self, $self->{db});
+    Tangram::Expr::Select->new(@_)->execute($self, $self->{db});
 }
 
 my $psi = 1;
@@ -699,8 +679,6 @@ sub _insert
 
 	use integer;
 
-    my @unhappy_ending;
-
 	for my $i (0..$#$sths) {
 
 	  if ($Tangram::TRACE) {
@@ -716,31 +694,13 @@ sub _insert
 	  my @args = (map {( ref $_ ? "$_" : $_ )} @state[ @{ $fields[$i] } ]);
 	  #print STDERR "args are: ".Data::Dumper::Dumper(\@args);
 	  #kill 2, $$;
-
-      local $@;
-      eval {
-        unless ($sth->execute(@args)) {
-          push @unhappy_ending, $dbh->errstr;
-        }
-      };
-      if ($@) {
-        push @unhappy_ending, $@;
-      }
+	  $sth->execute(@args)
+	      or die $dbh->errstr;
 
 	  $sth->finish();
 	}
 
-    if (@unhappy_ending) {
-      $self->{set_id}->($obj, undef);
-
-      # We warn, as we seem to be eating the error message before it gets
-      # printed.
-      warn "Could not completely store object $classId: ".join(", ", @unhappy_ending);
-      die "Could not completely store object $classId: ".join(", ", @unhappy_ending);
-    }
-    else {
-      return $id;
-    }
+    return $id;
   }
 
 #############################################################################
@@ -978,15 +938,13 @@ sub load
     return map { scalar $self->load( $_ ) } @_ if wantarray;
 
     my $id = shift;
-    confess("Invalid object-id $id, must match /^\\d+\$/")
-        unless ($id =~ /^\d+$/);
     die if @_;
 
     return $self->{objects}{$id}
       if exists $self->{objects}{$id} && defined $self->{objects}{$id};
 
     my $class = $self->{schema}->classdef( $self->{id2class}{ int(substr($id, -$self->{cid_size})) } );
-    confess("Unable to get Tangram class, $id is an invalid object ID?") unless $class;
+
 	my $row = _fetch_object_state($self, $id, $class);
 
     my $obj = $self->read_object($id, $class->{name}, $row);
@@ -1090,7 +1048,7 @@ sub _fetch_object_state
     my $row;
     $sth->execute($self->{export_id}->($id)) &&
 	($row = $sth->fetchrow_arrayref())
-	    or confess "could not find $class->{name} object "
+	    or croak "could not find $class->{name} object "
 		.$self->{export_id}->($id)." (oid $id) in storage";
 
     my $state = [ @$row ] if $row;
@@ -1150,13 +1108,13 @@ sub select {
 sub cursor_object
   {
     my ($self, $class) = @_;
-    $self->{IMPLICIT}{$class} ||= Tangram::RDBObject->new($self, $class)
+    $self->{IMPLICIT}{$class} ||= Tangram::Expr::RDBObject->new($self, $class)
 }
 
 sub query_objects
 {
     my ($self, @classes) = @_;
-    map { Tangram::QueryObject->new(Tangram::RDBObject->new($self, $_)) } @classes;
+    map { Tangram::Expr::QueryObject->new(Tangram::Expr::RDBObject->new($self, $_)) } @classes;
 }
 
 sub remote
@@ -1188,9 +1146,9 @@ sub aggregate
     do {
 	$filter = $expr;
 	$expr = Tangram::Expr->new
-	    (Tangram::Number->instance,
+	    (Tangram::Type::Number->instance,
 	     '*', $filter->objects);
-    } if $expr->isa("Tangram::Filter");
+    } if $expr->isa("Tangram::Expr::Filter");
 
     my @data = $self->select(undef,
 			     ($filter ? (filter => $filter) : ()),
@@ -1575,29 +1533,4 @@ sub DESTROY
     }
 }
 
-package Tangram::Storage::Statement;
-
-sub new
-{
-    my $class = shift;
-    bless { @_ }, $class;
-}
-
-sub fetchrow
-{
-    return shift->{statement}->fetchrow;
-}
-
-sub close
-{
-    my $self = shift;
-
-    if ($self->{storage})
-    {
-	$self->{statement}->finish;
-	$self->{storage}->close_connection($self->{connection});
-	%$self = ();
-    }
-}
-
 1;
